import axios from "axios";
import { parse, serialize } from "cookie";
import { NextApiRequest, NextApiResponse } from "next";
import { IAfOptions } from "./types";
import jwt_decode from "jwt-decode";

async function login(
  req: NextApiRequest,
  res: NextApiResponse,
  authConfig: IAfOptions
) {
  if (req.method === "POST") {
    const { email, password } = req.body;

    try {
      const apiResponse = await axios.post(
        `${authConfig.loginApiUrl}`,
        { email: email, password: password },

        {
          headers: {
            Accept: "application/json",
            "Content-Type": "application/json",
          },
        }
      );
      if (apiResponse.status === 200) {
        // Be carfule what is backend returning! here is data.access
        const accessDecode = jwt_decode(apiResponse.data.access);
        const refreshDecode = jwt_decode(apiResponse.data.refresh);

        res.setHeader("Set-Cookie", [
          serialize("access", apiResponse.data.access, {
            httpOnly: true,
            secure: process.env.NODE_ENV === "production" ? true : false,
            maxAge:
              !authConfig.expFromJwt || !(accessDecode as any).exp
                ? authConfig.accessTokenExpiryTime
                : undefined,
            expires:
              authConfig.expFromJwt && (accessDecode as any).exp
                ? new Date((accessDecode as any).exp * 1000)
                : undefined,
            sameSite: "strict",
            path: "/",
          }),
          serialize("refresh", apiResponse.data.refresh, {
            httpOnly: true,
            secure: process.env.NODE_ENV === "production" ? true : false,
            maxAge:
              !authConfig.expFromJwt || !(refreshDecode as any).exp
                ? authConfig.refreshTokenExpiryTime
                : undefined,
            expires:
              authConfig.expFromJwt && (refreshDecode as any).exp
                ? new Date((refreshDecode as any).exp * 1000)
                : undefined,
            sameSite: "strict",
            path: "/",
          }),
        ]);
        return res.status(200).json({ success: "Logged in succesfully" });
      } else {
        return res.status(500).json({ error: "Authentication failed" });
      }
    } catch (error) {
      return res
        .status((error as any).response.status)
        .json({ detail: (error as any).response.data.detail });
    }
  } else {
    res.setHeader("Allow", ["POST"]);
    return res.status(405).json({ error: `Methond ${req.method} not allowed` });
  }
}

async function logout(req: NextApiRequest, res: NextApiResponse) {
  if (req.method === "POST") {
    res.setHeader("Set-Cookie", [
      serialize("access", "", {
        httpOnly: true,
        secure: process.env.NODE_ENV === "production" ? true : false,
        expires: new Date(0),
        sameSite: "strict",
        path: "/",
      }),
    ]);
    return res
      .status(200)
      .json({ success: "Successfully cleared access token" });
  } else {
    res.setHeader("Allow", ["POST"]);
    return res.status(405).json({ error: `Methond ${req.method} not allowed` });
  }
}

async function verify(
  req: NextApiRequest,
  res: NextApiResponse,
  authConfig: IAfOptions
) {
  if (req.method === "POST") {
    const cookies = parse(req.headers.cookie || "");
    try {
      const apiResponse = await axios.post(
        `${authConfig.tokenValidityUrl}`,
        { token: cookies.access ? cookies.access : "none" },

        {
          headers: {
            Accept: "application/json",
            "Content-Type": "application/json",
          },
        }
      );
      if (apiResponse.status === 200) {
        return res.status(200).json({ tokenValid: true });
      } else if (apiResponse.status === 401) {
        return res.status(200).json({ tokenValid: false });
      } else {
        return res.status(apiResponse.status).json(apiResponse.data);
      }
    } catch (error) {
      // console.dir(error)
      if ((error as any).response.status == 401) {
        return res.status(200).json({ tokenValid: false });
      }
      return res
        .status((error as any).response.status)
        .json({ detail: (error as any).response.data.detail });
    }
  } else {
    res.setHeader("Allow", ["POST"]);
    return res.status(405).json({ error: `Methond ${req.method} not allowed` });
  }
}

async function AfAuthHandler(
  req: NextApiRequest,
  res: NextApiResponse,
  authConfig: IAfOptions
) {
  const { afAuth } = req.query;
  if (afAuth?.length == 1) {
    if (afAuth[0] == "login") {
      return await login(req, res, authConfig);
    } else if (afAuth[0] == "logout") {
      return await logout(req, res);
    } else if (afAuth[0] == "verify") {
      return await verify(req, res, authConfig);
    }
  }

  res.end(`Request on ${afAuth}`);
}

function AfAuth(
  ...args: [IAfOptions] | [NextApiRequest, NextApiResponse, IAfOptions]
) {
  if (args.length === 1) {
    return async (req: any, res: any) => await AfAuthHandler(req, res, args[0]);
  } else {
    return async (req: any, res: any) => await AfAuthHandler(req, res, args[2]);
  }
}

export default AfAuth;
